@@ -24,7 +24,7 @@
 #include "clamav-config.h"
 #endif
 
-static	char	const	rcsid[] = "$Id: tnef.c,v 1.26 2005/07/11 15:01:40 nigelhorne Exp $";
+static	char	const	rcsid[] = "$Id: tnef.c,v 1.31 2006/03/14 11:39:43 nigelhorne Exp $";
 
 #include <stdio.h>
 #include <fcntl.h>
@@ -38,8 +38,8 @@ static	char	const	rcsid[] = "$Id: tnef.c,v 1.26 2005/07/11 15:01:40 nigelhorne E
 #endif
 #include "blob.h"
 
-static	int	tnef_message(FILE *fp, uint16_t type, uint16_t tag, int32_t length);
-static	int	tnef_attachment(FILE *fp, uint16_t type, uint16_t tag, int32_t length, const char *dir, fileblob **fbref);
+static	int	tnef_message(FILE *fp, uint16_t type, uint16_t tag, int32_t length, off_t fsize);
+static	int	tnef_attachment(FILE *fp, uint16_t type, uint16_t tag, int32_t length, const char *dir, fileblob **fbref, off_t fsize);
 static	int	tnef_header(FILE *fp, uint8_t *part, uint16_t *type, uint16_t *tag, int32_t *length);
 
 #define	TNEF_SIGNATURE	0x223E9f78
@@ -73,9 +73,17 @@ cli_tnef(const char *dir, int desc)
 	fileblob *fb;
 	int i, ret, alldone;
 	FILE *fp;
+	off_t fsize;
+	struct stat statb;
 
 	lseek(desc, 0L, SEEK_SET);
 
+	if(fstat(desc, &statb) < 0) {
+		cli_errmsg("Can't fstat descriptor %d\n", desc);
+		return CL_EIO;
+	}
+	fsize = statb.st_size;
+
 	i = dup(desc);
 	if((fp = fdopen(i, "rb")) == NULL) {
 		cli_errmsg("Can't open descriptor %d\n", desc);
@@ -102,9 +110,9 @@ cli_tnef(const char *dir, int desc)
 	alldone = 0;
 
 	do {
-		uint8_t part;
-		uint16_t type, tag;
-		int32_t length;
+		uint8_t part = 0;
+		uint16_t type = 0, tag = 0;
+		int32_t length = 0;
 
 		switch(tnef_header(fp, &part, &type, &tag, &length)) {
 			case 0:
@@ -121,6 +129,13 @@ cli_tnef(const char *dir, int desc)
 				alldone = 1;
 				break;
 		}
+		if(length == 0)
+			continue;
+		if(length < 0) {
+			cli_warnmsg("Corrupt TNEF header detected - length %d\n", length);
+			ret = CL_EFORMAT;
+			break;
+		}
 		if(alldone)
 			break;
 		switch(part) {
@@ -131,7 +146,7 @@ cli_tnef(const char *dir, int desc)
 					fb = NULL;
 				}
 				fb = fileblobCreate();
-				if(tnef_message(fp, type, tag, length) != 0) {
+				if(tnef_message(fp, type, tag, length, fsize) != 0) {
 					cli_errmsg("Error reading TNEF message\n");
 					ret = CL_EFORMAT;
 					alldone = 1;
@@ -139,7 +154,7 @@ cli_tnef(const char *dir, int desc)
 				break;
 			case LVL_ATTACHMENT:
 				cli_dbgmsg("TNEF - found attachment\n");
-				if(tnef_attachment(fp, type, tag, length, dir, &fb) != 0) {
+				if(tnef_attachment(fp, type, tag, length, dir, &fb, fsize) != 0) {
 					cli_errmsg("Error reading TNEF message\n");
 					ret = CL_EFORMAT;
 					alldone = 1;
@@ -149,7 +164,7 @@ cli_tnef(const char *dir, int desc)
 				break;
 			default:
 				cli_warnmsg("TNEF - unknown level %d tag 0x%x\n", (int)part, (int)tag);
-				
+
 				/*
 				 * Dump the file incase it was part of an
 				 * email that's about to be deleted
@@ -188,7 +203,7 @@ cli_tnef(const char *dir, int desc)
 	if(fb) {
 		cli_dbgmsg("cli_tnef: flushing final data\n");
 		if(fileblobGetFilename(fb) == NULL) {
-			cli_dbgmsg("Saving TNEF portion with an unknown name");
+			cli_dbgmsg("Saving TNEF portion with an unknown name\n");
 			fileblobSetFilename(fb, dir, "tnef");
 		}
 		fileblobDestroy(fb);
@@ -200,10 +215,10 @@ cli_tnef(const char *dir, int desc)
 }
 
 static int
-tnef_message(FILE *fp, uint16_t type, uint16_t tag, int32_t length)
+tnef_message(FILE *fp, uint16_t type, uint16_t tag, int32_t length, off_t fsize)
 {
 	uint16_t i16;
-	/* off_t offset; */
+	off_t offset;
 #if	CL_DEBUG
 	uint32_t i32;
 	char *string;
@@ -211,7 +226,7 @@ tnef_message(FILE *fp, uint16_t type, uint16_t tag, int32_t length)
 
 	cli_dbgmsg("message tag 0x%x, type 0x%x, length %d\n", tag, type, length);
 
-	/* offset = ftell(fp); */
+	offset = ftell(fp);
 
 	/*
 	 * a lot of this stuff should be only discovered in debug mode...
@@ -261,27 +276,31 @@ tnef_message(FILE *fp, uint16_t type, uint16_t tag, int32_t length)
 
 	/*cli_dbgmsg("%lu %lu\n", (long)(offset + length), ftell(fp));*/
 
-	/* fseek(fp, offset + length, SEEK_SET); */
+	if(!CLI_ISCONTAINED2(0, fsize, (off_t)offset, (off_t)length)) {
+		cli_errmsg("TNEF: Incorrect length field\n");
+		return -1;
+	}
+	if(fseek(fp, offset + length, SEEK_SET) < 0)
+		return -1;
 
 	/* Checksum - TODO, verify */
-	/* if(fread(&i16, sizeof(uint16_t), 1, fp) != 1)
+	if(fread(&i16, sizeof(uint16_t), 1, fp) != 1)
 		return -1;
-	 */
 
 	return 0;
 }
 
 static int
-tnef_attachment(FILE *fp, uint16_t type, uint16_t tag, int32_t length, const char *dir, fileblob **fbref)
+tnef_attachment(FILE *fp, uint16_t type, uint16_t tag, int32_t length, const char *dir, fileblob **fbref, off_t fsize)
 {
 	uint32_t todo;
 	uint16_t i16;
-/*	off_t offset; */
+	off_t offset;
 	char *string;
 
 	cli_dbgmsg("attachment tag 0x%x, type 0x%x, length %d\n", tag, type, length);
 
-	/* offset = ftell(fp); */
+	offset = ftell(fp);
 
 	switch(tag) {
 		case attATTACHTITLE:
@@ -337,12 +356,16 @@ tnef_attachment(FILE *fp, uint16_t type, uint16_t tag, int32_t length, const cha
 
 	/*cli_dbgmsg("%lu %lu\n", (long)(offset + length), ftell(fp));*/
 
-	/* fseek(fp, (long)(offset + length), SEEK_SET); */	/* shouldn't be needed */
+	if(!CLI_ISCONTAINED2(0, fsize, (off_t)offset, (off_t)length)) {
+		cli_errmsg("TNEF: Incorrect length field\n");
+		return -1;
+	}
+	if(fseek(fp, (long)(offset + length), SEEK_SET) < 0)	/* shouldn't be needed */
+		return -1;
 
 	/* Checksum - TODO, verify */
-	/* if(fread(&i16, sizeof(uint16_t), 1, fp) != 1)
+	if(fread(&i16, sizeof(uint16_t), 1, fp) != 1)
 		return -1;
-	 */
 
 	return 0;
 }
@@ -362,8 +385,8 @@ tnef_header(FILE *fp, uint8_t *part, uint16_t *type, uint16_t *tag, int32_t *len
 		return -1;
 
 	i32 = host32(i32);
-	*tag = i32 & 0xFFFF;
-	*type = (i32 & 0xFFFF0000) >> 16;
+	*tag = (uint16_t)(i32 & 0xFFFF);
+	*type = (uint16_t)((i32 & 0xFFFF0000) >> 16);
 
 	if(fread(&i32, sizeof(uint32_t), 1, fp) != 1)
 		return -1;
